// 
// Beispielprogramm iws.c fuer die Benutzung des LP-Solvers LP-Solve
//
// Compilieren mit:    g++ beispiel.cc -o beispiel -L. -lm -llpk
//        
// Bemerkung:   Der Schalter -L. fuegt das aktuelle Verzeichnis
//              zur Liste der Pfade hinzu, in denen Libraries gesucht
//              werden. Steht die Datei liblpk.a in einem anderem 
//              Verzeichnis, als dem aktuellen, so ist der obige
//              Aufruf entsprechend zu modifizieren.
//
// Sven O. Krumke    07. Januar 1998
//
// 

// Fuer C++ angepasste Header stehen in lp_iws.h

#include "lp_iws.h"

#include <stdio.h>

void loesung(lprec *lp);

int main(void) 
{
  REAL zeile[3];
  lprec *lp;               // Datenstruktur fuer ein LP

  lp=make_lp(0,2);         // Datenstruktur initialisieren:
                           // 0 Constraints, 2 Variablen

  // Zielfunktion: x1 + 10*x2 (soll minimiert werden)

  zeile[1]=1;
  zeile[2]=10;
  set_obj_fn(lp,zeile);
  set_minim(lp);

  // Wahlweise:  str_set_obj_fn(lp1, "1 10");


  // Constraints:  66*x1 + 14*x2 >= 1430
  //              -82*x1 + 28*x2 >= 1306;

  // Constraint-Typen: LE  <=
  //                   GE  >=
  //                   EQ  ==

  zeile[1]=66;
  zeile[2]=14;
  add_constraint(lp,zeile,GE,1430);

  // Wahlweise:  str_add_constraint(lp1, "66 14",GE,1430);

  zeile[1]=-82;
  zeile[2]=28;
  add_constraint(lp,zeile,GE,1306);

  // Wahlweise:  str_add_constraint(lp1, "-82 28",GE,1306);


  // Variablentyp auf INTEGER fuer beide Variablen

  set_int(lp,1,TRUE);
  set_int(lp,2,TRUE);


  // LP anzeigen

  print_lp(lp);

  // LP loesen

  loesung(lp);

  // Wir ersetzen die >= in den Constraints durch ein =
  // und loesen wieder.

  set_constr_type(lp,1,EQ);
  set_constr_type(lp,2,EQ);
  print_lp(lp);
  loesung(lp);
  
}



void loesung(lprec *lp) 
{
  int ergebnis;

  ergebnis = solve(lp);

  // Solve Status-Meldungen
  // ----------------------
  //  OPTIMAL     	0
  //  MILP_FAIL   	1
  //  INFEASIBLE  	2
  //  UNBOUNDED   	3
  //  FAILURE     	4
  //  RUNNING     	5

  switch (ergebnis) 
    {
    case OPTIMAL :  
      printf ("Optimum wurde gefunden\n");
      printf ("Zielfunktion: %4.f\n", lp->best_solution[0]);
      printf ("Constraint 1: %4.f\n", lp->best_solution[1]);
      printf ("Constraint 2: %4.f\n", lp->best_solution[2]);
      printf ("Loesung     : %f %f\n\n",lp->best_solution[3],
	      lp->best_solution[4]);

      // Wahlweise:   print_solution(lp);
      break;

    case INFEASIBLE :
      printf ("Problem besitzt keine zulaessige Loesung\n\n");
      break;

    default:
      printf ("Loesung nicht erfolgreich, Code = %d\n\n",ergebnis);
    }

}
